<?php
/**
 * Set User Role
 *
 * @package     AutomatorWP\Integrations\BuddyBoss\Actions\Set_User_Role
 * @author      AutomatorWP <contact@automatorwp.com>, Ruben Garcia <rubengcdev@gmail.com>
 * @since       1.0.0
 */
// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

class AutomatorWP_BuddyBoss_Set_User_Role extends AutomatorWP_Integration_Action {

    public $integration = 'buddyboss';
    public $action = 'buddyboss_set_user_role';

    /**
     * Store the action result
     *
     * @since 1.0.0
     *
     * @var string $result
     */
    public $result = '';

    /**
     * Register the trigger
     *
     * @since 1.0.0
     */
    public function register() {

        automatorwp_register_action( $this->action, array(
            'integration'       => $this->integration,
            'label'             => __( 'Set role to user in group', 'automatorwp-pro' ),
            'select_option'     => __( 'Set role to user in <strong>group</strong>', 'automatorwp-pro' ),
            /* translators: %1$s: Role. %2$s: Group. */
            'edit_label'        => sprintf( __( 'Set %1$s role to user in %2$s', 'automatorwp-pro' ), '{role}', '{group}' ),
            /* translators: %1$s: Group. */
            'log_label'         => sprintf( __( 'Set role to user in %1$s', 'automatorwp-pro' ), '{group}' ),
            'options'           => array(
                'group' => automatorwp_utilities_ajax_selector_option( array(
                    'field'             => 'group',
                    'option_default'    => __( 'group', 'automatorwp-pro' ),
                    'name'              => __( 'Group:', 'automatorwp-pro' ),
                    'option_none'       => false,
                    'option_custom'         => true,
                    'option_custom_desc'    => __( 'Group ID', 'automatorwp-pro' ),
                    'action_cb'         => 'automatorwp_buddyboss_get_groups',
                    'options_cb'        => 'automatorwp_buddyboss_options_cb_group',
                    'placeholder'       => __( 'Select a group', 'automatorwp-pro' ),
                    'default'           => '',
                ) ),
                'role' => array(
                    'default' => __( 'choose role', 'automatorwp-pro' ),
                    'from' => 'role',
                    'fields' => array(
                        'role' => array(
                            'name' => __( 'Roles:', 'automatorwp-pro' ),
                            'type' => 'select',
                            'options' => array(
                                'admin' => 'Organizer',
                                'mod' => 'Moderator',
                                'member'    => 'Member',
                                'banned'    =>'Banned',
                            ),
                        ),
                    )
                ),
            ),
        ) );

    }

    /**
     * Action execution function
     *
     * @since 1.0.0
     *
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     */
    public function execute( $action, $user_id, $action_options, $automation ) {

        $this->result = '';

        // Bail if BuddyBoss function does not exist
        if ( ! function_exists( 'bp_get_user_group_role_title' ) ) {
            $this->result = __( 'Groups component is not active.', 'automatorwp-pro' );
            return;
        }

        // Shorthand
        $group_id = absint( $action_options['group'] );
        $role = $action_options['role'];
        $is_banned = false;

        // Bail if group not provided
        if( absint( $group_id ) === 0 ) {
            return;
        }

        $member = new BP_Groups_Member($user_id, $group_id);

        $user_role = bp_get_user_group_role_title( $user_id, $group_id );
        
        // Bail if user does not member of group or is banned
        if( empty( $user_role ) ) {
            if ( groups_is_user_banned( $user_id, $group_id ) ) {
                $is_banned = true;
            } else {
                $this->result = __( 'User does not belong to group.', 'automatorwp-pro' );
                return;
            }
            
        }
       
        $buddyboss_roles = array(
            'admin'     => 'Organizer',
            'mod'       => 'Moderator',
            'member'    => 'Member',
            'banned'    => 'Banned',
        );
        
        // Bail if user role is the same
        if ( $user_role === $buddyboss_roles[$role] ) {
            $this->result = sprintf ( __( 'The user already had the role %s.', 'automatorwp-pro' ), $buddyboss_roles[$role] );
            return;
        }

        // Bail if user role is the same
        if ( $is_banned && $buddyboss_roles[$role] === 'Banned' ) {
            $this->result = __( 'The user already had the role banned.', 'automatorwp-pro' );
            return;
        }

        switch ( $role ) {
            case 'admin':
            case 'mod':
                if ( $is_banned ) {
                    $member->unban();
                    $member = new BP_Groups_Member($user_id, $group_id);
                }
                $member->promote( $role );
                break;
            case 'member':
                if ( $is_banned ) {
                    $member->unban();
                    $member = new BP_Groups_Member($user_id, $group_id);
                }
                $member->demote();
                break;
            case 'banned':
                if ( groups_is_user_admin( $user_id, $group_id ) ) {
                    $member->demote();
                    $member = new BP_Groups_Member($user_id, $group_id);
                }
                $member->ban();
                break;
        }
       
        // Change the user role to the group
        $this->result = sprintf ( __( 'User changes role to %s.', 'automatorwp-pro' ), $buddyboss_roles[$role] );

    }

    /**
     * Register required hooks
     *
     * @since 1.0.0
     */
    public function hooks() {

        // Log meta data
        add_filter( 'automatorwp_user_completed_action_log_meta', array( $this, 'log_meta' ), 10, 5 );

        // Log fields
        add_filter( 'automatorwp_log_fields', array( $this, 'log_fields' ), 10, 5 );

        parent::hooks();

    }    

    /**
     * Action custom log meta
     *
     * @since 1.0.0
     *
     * @param array     $log_meta           Log meta data
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     *
     * @return array
     */
    public function log_meta( $log_meta, $action, $user_id, $action_options, $automation ) {

        // Bail if action type don't match this action
        if( $action->type !== $this->action ) {
            return $log_meta;
        }

        $log_meta['result'] = $this->result;

        return $log_meta;

    }

    /**
     * Action custom log fields
     *
     * @since 1.0.0
     *
     * @param array     $log_fields The log fields
     * @param stdClass  $log        The log object
     * @param stdClass  $object     The trigger/action/automation object attached to the log
     *
     * @return array
     */
    public function log_fields( $log_fields, $log, $object ) {

        // Bail if log is not assigned to an action
        if( $log->type !== 'action' ) {
            return $log_fields;
        }

        // Bail if action type don't match this action
        if( $object->type !== $this->action ) {
            return $log_fields;
        }

        $log_fields['result'] = array(
            'name' => __( 'Result:', 'automatorwp-pro' ),
            'type' => 'text',
        );

        return $log_fields;

    }    

}

new AutomatorWP_BuddyBoss_Set_User_Role();