<?php
/**
 * Add Contact To List
 *
 * @package     AutomatorWP\Integrations\Newsletter\Actions\Add_Contact_To_List
 * @author      AutomatorWP <contact@automatorwp.com>, Ruben Garcia <rubengcdev@gmail.com>
 * @since       1.0.0
 */
// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

class AutomatorWP_Newsletter_Add_Contact_To_List extends AutomatorWP_Integration_Action {

    public $integration = 'newsletter';
    public $action = 'newsletter_add_contact_to_list';

    /**
     * The action result
     *
     * @since 1.0.0
     *
     * @var string $result
     */
    public $result = '';

    /**
     * Register the trigger
     *
     * @since 1.0.0
     */
    public function register() {

        automatorwp_register_action( $this->action, array(
            'integration'       => $this->integration,
            'label'             => __( 'Add contact to a list', 'automatorwp-pro' ),
            'select_option'     => __( 'Add contact to a <strong>list</strong>', 'automatorwp-pro' ),
            /* translators: %1$s: Contact. %2$s: List. */
            'edit_label'        => sprintf( __( 'Add %1$s to %2$s', 'automatorwp-pro' ), '{contact}', '{list}' ),
            /* translators: %1$s: Contact. %2$s: List. */
            'log_label'         => sprintf( __( 'Add %1$s to %2$s', 'automatorwp-pro' ), '{contact}', '{list}' ),
            'options'           => array(
                'contact' => array(
                    'from' => 'email',
                    'default' => __( 'contact', 'automatorwp-pro' ),
                    'fields' => array(
                        'email' => array(
                            'name' => __( 'Email:', 'automatorwp-pro' ),
                            'desc' => __( 'The contact email address.', 'automatorwp-pro' ),
                            'type' => 'text',
                            'required'  => true,
                            'default' => ''
                        ),
                        'name' => array(
                            'name' => __( 'Name:', 'automatorwp-pro' ),
                            'desc' => __( 'The contact name.', 'automatorwp-pro' ),
                            'type' => 'text',
                            'default' => ''
                        ),
                     ) ),
                'list' => array(
                    'from' => 'list',
                    'fields' => array(
                        'list' => array(
                            'name' => __( 'List:', 'automatorwp-pro' ),
                            'type' => 'select',
                            'classes' => 'automatorwp-selector',
                            'options_cb' => array( $this, 'lists_options_cb' ),
                            'default' => 'any'
                        ),
                        'status' => array(
                            'name' => __( 'Status:', 'automatorwp-pro' ),
                            'type' => 'select',
                            'options' => array(
                                'confirmed'    => __( 'Confirmed', 'automatorwp-pro' ),
                                'notconfirmed'   => __( 'Not confirmed', 'automatorwp-pro' ),
                            ),
                            'default' => 'confirmed'
                        ),
                    )
                ),
            ),
        ) );

    }

    /**
     * Get lists options
     *
     * @since 1.0.0
     *
     * @return array
     */
    public function lists_options_cb() {

        $options = array(
            'any' => __( 'any list', 'automatorwp-pro' ),
        );

        if ( class_exists( '\Newsletter' ) ) {

            $lists = \Newsletter::instance()->get_lists();

            if ( ! empty( $lists ) ) {
                foreach ( $lists as $list ) {
                    $options['list_' . $list->id] = $list->name;
                }
            }

        }

        return $options;

    }

    /**
     * Action execution function
     *
     * @since 1.0.0
     *
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     */
    public function execute( $action, $user_id, $action_options, $automation ) {

        $this->result = '';

        // Shorthand
        $list_id = $action_options['list'];
        $status = $action_options['status'];
        $contact_email = $action_options['email'];
        $contact_name = $action_options['name'];

        // Bail if not list selected
        if( $list_id === '' ) {
            $this->result = __( 'No list provided.', 'automatorwp-pro' );
            return;
        }

        if ( ! class_exists( '\NewsletterUsers' ) ) {
            $this->result = __( 'Class NewsletterUsers not found.', 'automatorwp-pro' );
            return;
        }

        $status = ($status === 'confirmed') ? \TNP_User::STATUS_CONFIRMED : \TNP_User::STATUS_NOT_CONFIRMED;;
        
        // Setup the NewsletterUsers instance to save the user data
        $newsletter_users = \NewsletterUsers::instance();

        $subscriber_data = $newsletter_users->get_user( $contact_email );

        // Create if not exists
        if ( empty( $subscriber_data ) ) {
			$subscriber_data = (object) array(
				'email'         => $contact_email,
				'name'          => $contact_name,
                'status'        => $status,
			);
		}

        // Subscribe user to the list
        $subscriber_data->$list_id = 1;
        
        $subscriber = $newsletter_users->save_user( $subscriber_data );

        if( $subscriber ) {
            $this->result = __( 'User added to list successfully.', 'automatorwp-pro' );
        } else {
            $this->result = __( 'User not added to list.', 'automatorwp-pro' );
        }

    }

    /**
     * Register required hooks
     *
     * @since 1.0.0
     */
    public function hooks() {

        // Log meta data
        add_filter( 'automatorwp_user_completed_action_log_meta', array( $this, 'log_meta' ), 10, 5 );

        // Log fields
        add_filter( 'automatorwp_log_fields', array( $this, 'log_fields' ), 10, 5 );

        parent::hooks();

    }

    /**
     * Action custom log meta
     *
     * @since 1.0.0
     *
     * @param array     $log_meta           Log meta data
     * @param stdClass  $action             The action object
     * @param int       $user_id            The user ID
     * @param array     $action_options     The action's stored options (with tags already passed)
     * @param stdClass  $automation         The action's automation object
     *
     * @return array
     */
    public function log_meta( $log_meta, $action, $user_id, $action_options, $automation ) {

        // Bail if action type don't match this action
        if( $action->type !== $this->action ) {
            return $log_meta;
        }

        $log_meta['list_id'] = $action_options['list'];

        // Store result
        $log_meta['result'] = $this->result;

        return $log_meta;
    }

    /**
     * Action custom log fields
     *
     * @since 1.0.0
     *
     * @param array     $log_fields The log fields
     * @param stdClass  $log        The log object
     * @param stdClass  $object     The trigger/action/automation object attached to the log
     *
     * @return array
     */
    public function log_fields( $log_fields, $log, $object ) {

        // Bail if log is not assigned to an action
        if( $log->type !== 'action' ) {
            return $log_fields;
        }

        // Bail if action type don't match this action
        if( $object->type !== $this->action ) {
            return $log_fields;
        }

        $log_fields['list_id'] = array(
            'name' => __( 'List subscribed', 'automatorwp-pro' ),
            'desc' => __( 'List subscribed.', 'automatorwp-pro' ),
            'type' => 'text',
            'before_field' => array( $this, 'list_display_cb' ),
        );

        $log_fields['result'] = array(
            'name' => __( 'Result:', 'automatorwp-pro' ),
            'type' => 'text',
        );

        return $log_fields;
    }

    /**
     * Callback used to render list on logs
     *
     * @since 1.0.0
     *
     * @param array         $field_args
     * @param CMB2_Field    $field
     */
    public function list_display_cb( $field_args, $field ) {

        $list_id = $field->value();

        $lists = $this->lists_options_cb();

        echo ( isset( $lists[$list_id] ) ? $lists[$list_id] : sprintf( __( 'List with ID %s not found.', 'automatorwp-pro' ), $list_id ) );

    }

}

new AutomatorWP_Newsletter_Add_Contact_To_List();